mod events;

use std::fs::{File, OpenOptions};
use std::io;
use std::os::fd::OwnedFd;
use std::os::unix::fs::OpenOptionsExt;
use std::path::Path;

use input::{Libinput, LibinputInterface};
use libc::{O_RDONLY, O_RDWR, O_WRONLY};

const UDEV_SEAT: &str = "seat0";

struct Interface;

impl LibinputInterface for Interface {
    fn open_restricted(&mut self, path: &Path, flags: i32) -> Result<OwnedFd, i32> {
        OpenOptions::new()
            .custom_flags(flags)
            .read((flags & O_RDONLY == O_RDONLY) | (flags & O_RDWR == O_RDWR))
            .write((flags & O_WRONLY == O_WRONLY) | (flags & O_RDWR == O_RDWR))
            .open(path)
            .map(|file| file.into())
            .map_err(|err| err.raw_os_error().unwrap())
    }

    fn close_restricted(&mut self, fd: OwnedFd) {
        File::from(fd);
    }
}

pub(crate) fn read_inputs() -> io::Result<()> {
    let mut input = Libinput::new_with_udev(Interface);
    input.udev_assign_seat(UDEV_SEAT).unwrap();
    loop {
        input.dispatch().unwrap();
        for event in &mut input {
            // event.
            println!("Got event: {:?}", event);
        }
    }
}
